/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.concurrent.Executor;


import com.redhat.rhevm.api.model.DataCenter;
import com.redhat.rhevm.api.model.Cluster;
import com.redhat.rhevm.api.model.CPU;
import com.redhat.rhevm.api.resource.AssignedNetworksResource;
import com.redhat.rhevm.api.resource.AssignedPermissionsResource;
import com.redhat.rhevm.api.resource.ClusterResource;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;


public class MockClusterResource extends AbstractMockResource<Cluster> implements ClusterResource {

    /**
     * Package-protected ctor, never needs to be instantiated by JAX-RS framework.
     *
     * @param cluster  encapsulated Cluster
     * @param executor    executor used for asynchronous actions
     */
    MockClusterResource(String id, Executor executor, UriInfoProvider uriProvider) {
        super(id, executor, uriProvider);
    }

    // FIXME: this needs to be atomic
    public void updateModel(Cluster cluster) {
        // update writable fields only
        if (cluster.isSetName()) {
            getModel().setName(cluster.getName());
        }
        if (cluster.isSetDescription()) {
            getModel().setDescription(cluster.getDescription());
        }

        if (cluster.isSetCpu()) {
            CPU cpu = new CPU();
            cpu.setId(cluster.getCpu().getId());
            getModel().setCpu(cpu);
        }

        if (cluster.isSetDataCenter()) {
            DataCenter dataCenter = new DataCenter();
            dataCenter.setId(cluster.getDataCenter().getId());
            getModel().setDataCenter(dataCenter);
        }
    }

    public Cluster addLinks() {
        return LinkHelper.addLinks(getUriInfo(), JAXBHelper.clone(OBJECT_FACTORY.createCluster(getModel())));
    }

    @Override
    public Cluster get() {
        return addLinks();
    }

    @Override
    public Cluster update(Cluster cluster) {
        validateUpdate(cluster);
        updateModel(cluster);
        return addLinks();
    }

    @Override
    public AssignedNetworksResource getAssignedNetworksSubResource() {
        return null;
    }

    @Override
    public AssignedPermissionsResource getPermissionsResource() {
        return null;
    }
}
