/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.Action;
import com.redhat.rhevm.api.model.DataCenter;
import com.redhat.rhevm.api.model.DataCenters;
import com.redhat.rhevm.api.model.StorageType;
import com.redhat.rhevm.api.resource.DataCenterResource;
import com.redhat.rhevm.api.resource.DataCentersResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockDataCentersResource extends AbstractMockQueryableResource<DataCenter> implements DataCentersResource {

    private static Map<String, MockDataCenterResource> dataCenters =
        Collections.synchronizedMap(new HashMap<String, MockDataCenterResource>());

    public MockDataCentersResource() {
        super(new SimpleQueryEvaluator<DataCenter>());
    }

    public void populate() {
        synchronized (dataCenters) {
            while (dataCenters.size() < 2) {
                MockDataCenterResource resource = new MockDataCenterResource(allocateId(DataCenter.class), getExecutor(), this);
                resource.getModel().setName("datacenter" + resource.getModel().getId());
                resource.getModel().setStorageType((dataCenters.size() % 2) == 0 ? StorageType.ISCSI.value() : StorageType.NFS.value());
                dataCenters.put(resource.getModel().getId(), resource);
            }
        }
    }

    @Override
    public DataCenters list() {
        DataCenters ret = new DataCenters();

        for (MockDataCenterResource dataCenter : dataCenters.values()) {
            if (filter(dataCenter.getModel(), getUriInfo(), DataCenter.class)) {
                ret.getDataCenters().add(dataCenter.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(DataCenter dataCenter) {
        MockDataCenterResource resource = new MockDataCenterResource(allocateId(DataCenter.class), getExecutor(), this);

        resource.updateModel(dataCenter);

        String id = resource.getId();
        dataCenters.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        dataCenter = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(dataCenter).build();
    }

    @Override
    public void remove(String id) {
        dataCenters.remove(id);
    }

    @Override
    public void remove(String id, Action action) {
        dataCenters.remove(id);
    }

    @Override
    public DataCenterResource getDataCenterSubResource(String id) {
        return dataCenters.get(id);
    }
}
