/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.Host;
import com.redhat.rhevm.api.model.Hosts;
import com.redhat.rhevm.api.resource.HostResource;
import com.redhat.rhevm.api.resource.HostsResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockHostsResource extends AbstractMockQueryableResource<Host> implements HostsResource {

    private static Map<String, MockHostResource> hosts =
        Collections.synchronizedMap(new HashMap<String, MockHostResource>());

    public MockHostsResource() {
        super(new SimpleQueryEvaluator<Host>());
    }

    public void populate() {
        synchronized (hosts) {
            while (hosts.size() < 4) {
                MockHostResource resource = new MockHostResource(allocateId(Host.class), getExecutor(), this);
                resource.getModel().setName("host" + resource.getModel().getId());
                hosts.put(resource.getModel().getId(), resource);
            }
        }
    }

    @Override
    public Hosts list() {
        Hosts ret = new Hosts();

        for (MockHostResource host : hosts.values()) {
            if (filter(host.getModel(), getUriInfo(), Host.class)) {
                ret.getHosts().add(host.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(Host host) {
        MockHostResource resource = new MockHostResource(allocateId(Host.class), getExecutor(), this);

        resource.updateModel(host);

        String id = resource.getId();
        hosts.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        host = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(host).build();
    }

    @Override
    public void remove(String id) {
        hosts.remove(id);
    }

    @Override
    public HostResource getHostSubResource(String id) {
        return hosts.get(id);
    }
}
