/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.model.Templates;
import com.redhat.rhevm.api.resource.TemplateResource;
import com.redhat.rhevm.api.resource.TemplatesResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockTemplatesResource extends AbstractMockQueryableResource<Template> implements TemplatesResource {

    private static Map<String, MockTemplateResource> templates =
        Collections.synchronizedMap(new HashMap<String, MockTemplateResource>());

    public MockTemplatesResource() {
        super(new SimpleQueryEvaluator<Template>());
    }

    public void populate() {
        synchronized (templates) {
            while (templates.size() < 4) {
                MockTemplateResource resource = new MockTemplateResource(allocateId(Template.class), getExecutor(), this);
                resource.getModel().setName("template" + resource.getModel().getId());
                templates.put(resource.getModel().getId(), resource);
            }
        }
    }

    @Override
    public Templates list() {
        Templates ret = new Templates();

        for (MockTemplateResource template : templates.values()) {
            if (filter(template.getModel(), getUriInfo(), Template.class)) {
                ret.getTemplates().add(template.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(Template template) {
        MockTemplateResource resource = new MockTemplateResource(allocateId(Template.class), getExecutor(), this);

        resource.updateModel(template);

        String id = resource.getId();
        templates.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        template = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(template).build();
    }

    @Override
    public void remove(String id) {
        templates.remove(id);
    }

    @Override
    public TemplateResource getTemplateSubResource(String id) {
        return templates.get(id);
    }
}
