/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.concurrent.Executor;

import com.redhat.rhevm.api.model.Cluster;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.model.VmPool;
import com.redhat.rhevm.api.resource.AssignedPermissionsResource;
import com.redhat.rhevm.api.resource.VmPoolResource;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.JAXBHelper;
import com.redhat.rhevm.api.common.util.LinkHelper;


public class MockVmPoolResource extends AbstractMockResource<VmPool> implements VmPoolResource {
    /* FIXME: would like to do:
     * private @Context UriInfo uriInfo;
     */

    /**
     * Package-protected ctor, never needs to be instantiated by JAX-RS framework.
     *
     * @param pool     encapsulated VmPool
     * @param executor executor used for asynchronous actions
     */
    MockVmPoolResource(String id, Executor executor, UriInfoProvider uriProvider) {
        super(id, executor, uriProvider);
    }

    // FIXME: this needs to be atomic
    public void updateModel(VmPool pool) {
        // update writable fields only
        if (pool.isSetName()) {
            getModel().setName(pool.getName());
        }
        if (pool.isSetDescription()) {
            getModel().setDescription(pool.getDescription());
        }
        if (pool.isSetCluster() && pool.getCluster().isSetId()) {
            getModel().setCluster(new Cluster());
            getModel().getCluster().setId(pool.getCluster().getId());
        }
        if (pool.isSetTemplate() && pool.getTemplate().isSetId()) {
            getModel().setTemplate(new Template());
            getModel().getTemplate().setId(pool.getTemplate().getId());
        }
    }

    public VmPool addLinks() {
        return LinkHelper.addLinks(getUriInfo(), JAXBHelper.clone("vmpool", VmPool.class, getModel()));
    }

    @Override
    public VmPool get() {
        return addLinks();
    }

    @Override
    public VmPool update(VmPool pool) {
        validateUpdate(pool);
        updateModel(pool);
        return addLinks();
    }

    @Override
    public AssignedPermissionsResource getPermissionsResource() {
        // TODO Auto-generated method stub
        return null;
    }
}
